package egg_energy

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	egg_system_rules "code.fnuoos.com/EggPlanet/egg_system_rules.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

// ActivityCoinReadyExchangeEggEnergy 蛋蛋积分【准备】兑换成蛋蛋能量
func ActivityCoinReadyExchangeEggEnergy(engine *xorm.Engine, uid int64, ch *rabbit.Channel) (err error) {
	//1、查找`egg_energy_basic_setting` 基础设置
	eggEnergyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(engine)
	eggEnergyBasicSetting, err := eggEnergyBasicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if eggEnergyBasicSetting.ExchangeRules == "" {
		err = errors.New("自动兑换未设置！")
		return
	}

	var exchangeRulesStruct *md2.ExchangeRulesStruct
	err = json.Unmarshal([]byte(eggEnergyBasicSetting.ExchangeRules), &exchangeRulesStruct)
	if err != nil {
		return
	}
	var videoRewardSystem *md2.VideoRewardSystemStruct
	err = json.Unmarshal([]byte(eggEnergyBasicSetting.VideoRewardSystem), &videoRewardSystem)
	if err != nil {
		return
	}
	if videoRewardSystem.RewardTotalNum == "" || videoRewardSystem.IntervalMinutes == "" || videoRewardSystem.EachRoundHour == "" {
		err = errors.New("视屏奖励机制设置未完全！")
		return
	}

	var autoExchangeNumsByPerson = egg_system_rules.StrToFloat64(exchangeRulesStruct.AutoExchangeNumsByPerson) //个人活跃积分X个自动兑换
	fmt.Println("autoExchangeNumsByPerson>>>>>>>>>>>>", autoExchangeNumsByPerson)
	var autoExchangeNumsByTeam = egg_system_rules.StrToFloat64(exchangeRulesStruct.AutoExchangeNumsByTeam) //个人活跃积分X个自动兑换
	fmt.Println("autoExchangeNumsByTeam>>>>>>>>>>>>", autoExchangeNumsByTeam)

	//2、获取"个人蛋蛋活跃积分" && 获取"团队蛋蛋活跃积分"
	var coin1, coin2 model.VirtualCoin
	_, err = engine.Where("id = ?", eggEnergyBasicSetting.PersonEggPointsCoinId).Get(&coin1)
	if err != nil {
		return err
	}
	_, err = engine.Where("id = ?", eggEnergyBasicSetting.TeamEggPointsCoinId).Get(&coin2)
	if err != nil {
		return err
	}
	personActivePointsCoinExchangeRatioValue, _ := decimal.NewFromString(coin1.ExchangeRatio)
	teamActivePointsCoinExchangeRatioValue, _ := decimal.NewFromString(coin2.ExchangeRatio)

	//3、当前 "个人活跃积分"可以自动兑换的用户数据 && "团队活跃积分"可以自动兑换的用户数据
	var personUserVirtualAmount model.UserVirtualAmount
	_, err = engine.Where("coin_id = ?", eggEnergyBasicSetting.PersonEggPointsCoinId).And("uid =?", uid).Get(&personUserVirtualAmount)
	if err != nil {
		fmt.Println("err:::::1111", err)
		return
	}
	var teamUserVirtualAmount model.UserVirtualAmount
	_, err = engine.Where("coin_id = ?", eggEnergyBasicSetting.TeamEggPointsCoinId).And("uid =?", uid).Get(&teamUserVirtualAmount)
	if err != nil {
		fmt.Println("err:::::2222", err)
		return
	}

	if egg_system_rules.StrToFloat64(personUserVirtualAmount.Amount) < autoExchangeNumsByPerson && egg_system_rules.StrToFloat64(teamUserVirtualAmount.Amount) < autoExchangeNumsByTeam {
		return errors.New("当前活跃值无法发起挖掘~")
	}

	//4、计算能兑换的个人活跃积分数量
	var autoExchangeNumsByPersonAmountValue, autoExchangeNumsByTeamAmountValue string
	var autoExchangeNumsByPersonAmount, autoExchangeNumsByTeamAmount decimal.Decimal
	if egg_system_rules.StrToFloat64(personUserVirtualAmount.Amount) >= autoExchangeNumsByPerson {
		autoExchangeNumsByPersonValue, _ := decimal.NewFromString(personUserVirtualAmount.Amount)
		autoExchangeRateByPersonValue, _ := decimal.NewFromString(exchangeRulesStruct.AutoExchangeRateByPerson)
		autoExchangeNumsByPersonAmountValue = autoExchangeNumsByPersonValue.Mul(autoExchangeRateByPersonValue).Div(decimal.NewFromInt(100)).String()
		autoExchangeNumsByPersonAmount = autoExchangeNumsByPersonValue.Mul(autoExchangeRateByPersonValue).Div(decimal.NewFromInt(100)).Div(personActivePointsCoinExchangeRatioValue) //个人活跃积分数量*兑换百分比/兑换比例(与金额)
	}

	//5、计算能兑换的团队活跃积分数量
	if egg_system_rules.StrToFloat64(teamUserVirtualAmount.Amount) >= autoExchangeNumsByTeam {
		autoExchangeNumsByTeamValue, _ := decimal.NewFromString(teamUserVirtualAmount.Amount)
		autoExchangeRateByTeamValue, _ := decimal.NewFromString(exchangeRulesStruct.AutoExchangeRateByTeam)
		autoExchangeNumsByTeamAmountValue = autoExchangeNumsByTeamValue.Mul(autoExchangeRateByTeamValue).Div(decimal.NewFromInt(100)).String()
		autoExchangeNumsByTeamAmount = autoExchangeNumsByTeamValue.Mul(autoExchangeRateByTeamValue).Div(decimal.NewFromInt(100)).Div(teamActivePointsCoinExchangeRatioValue) //团队活跃积分数量*兑换百分比/兑换比例(与金额)
	}

	//6、插入签到记录
	now := time.Now()
	eggSignInDb := implement.NewEggSignInDb(engine)
	var sign = &model.EggSignIn{
		Uid:                                   uid,
		StartTime:                             now.Format("2006-01-02 15:04:05"),
		EndTime:                               now.Add(time.Hour * time.Duration(egg_system_rules.StrToInt64(videoRewardSystem.EachRoundHour))).Format("2006-01-02 15:04:05"),
		TotalPersonEggPoints:                  autoExchangeNumsByPersonAmountValue,
		TotalTeamEggPoints:                    autoExchangeNumsByTeamAmountValue,
		EstimatePerSecondPersonEggEnergyValue: "",
		EstimatePerSecondTeamEggEnergyValue:   "",
		TotalPersonEggPointsAmountValue:       autoExchangeNumsByPersonAmount.String(),
		TotalTeamEggPointsAmountValue:         autoExchangeNumsByTeamAmount.String(),
		IsCompleted:                           0,
	}
	_, err = eggSignInDb.EggSignInInsert(sign)
	if err != nil {
		return err
	}

	//7、新增 `egg_energy_user_activity` 记录
	var m model.EggEnergyUserActivity
	has, err := engine.Where("uid =? and date =?", uid, now.Format("2006-01-02")).Get(&m)
	if err != nil {
		return err
	}
	if !has {
		eggEnergyUserActivityDb := implement.NewEggEnergyUserActivityDb(engine)
		insertAffected, err1 := eggEnergyUserActivityDb.EggEnergyUserActivityInsert(&model.EggEnergyUserActivity{
			Uid:  uid,
			Date: now.Format("2006-01-02"),
		})
		if err1 != nil {
			return err1
		}
		if insertAffected <= 0 {
			return errors.New("插入活跃记录失败")
		}
	}

	//TODO::推入rabbitmq 异步处理
	ch.Publish(md2.EggEnergyExchange, md2.EggEnergyStructForStartExchangeGreenEnergy{
		Uid:                   uid,
		SignId:                sign.Id,
		PersonEggEnergyCoinId: eggEnergyBasicSetting.PersonEggEnergyCoinId,
		TeamEggEnergyCoinId:   eggEnergyBasicSetting.TeamEggEnergyCoinId,
		PersonPointCoinId:     coin1.Id,
		PersonEnergyCoinId:    eggEnergyBasicSetting.PersonEggEnergyCoinId,
		TeamPointCoinId:       coin2.Id,
		TeamEnergyCoinId:      eggEnergyBasicSetting.TeamEggEnergyCoinId,
	}, md2.EggEnergyRoutKeyForStartExchangeGreenEnergy)
	return
}
